@@ -2,9 +2,9 @@
  * \file EllipticFunction.hpp
  * \brief Header for GeographicLib::EllipticFunction class
  *
- * Copyright (c) Charles Karney (2008-2012) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2008-2022) <karney@alum.mit.edu> and licensed
  * under the MIT/X11 License.  For more information, see
- * http://geographiclib.sourceforge.net/
+ * https://geographiclib.sourceforge.io/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_ELLIPTICFUNCTION_HPP)
@@ -21,10 +21,10 @@ namespace GeographicLib {
    * GeodesicExact, and TransverseMercatorExact.  Two categories of function
    * are provided:
    * - \e static functions to compute symmetric elliptic integrals
-   *   (http://dlmf.nist.gov/19.16.i)
+   *   (https://dlmf.nist.gov/19.16.i)
    * - \e member functions to compute Legrendre's elliptic
-   *   integrals (http://dlmf.nist.gov/19.2.ii) and the
-   *   Jacobi elliptic functions (http://dlmf.nist.gov/22.2).
+   *   integrals (https://dlmf.nist.gov/19.2.ii) and the
+   *   Jacobi elliptic functions (https://dlmf.nist.gov/22.2).
    * .
    * In the latter case, an object is constructed giving the modulus \e k (and
    * optionally the parameter &alpha;<sup>2</sup>).  The modulus is always
@@ -36,25 +36,26 @@ namespace GeographicLib {
    * In geodesic applications, it is convenient to separate the incomplete
    * integrals into secular and periodic components, e.g.,
    * \f[
-   *   E(\phi, k) = (2 E(\phi) / \pi) [ \phi + \delta E(\phi, k) ]
+   *   E(\phi, k) = (2 E(k) / \pi) [ \phi + \delta E(\phi, k) ]
    * \f]
    * where &delta;\e E(&phi;, \e k) is an odd periodic function with period
    * &pi;.
    *
    * The computation of the elliptic integrals uses the algorithms given in
    * - B. C. Carlson,
-   *   <a href="http://dx.doi.org/10.1007/BF02198293"> Computation of real or
-   *   complex elliptic integrals</a>, Numerical Algorithms 10, 13--26 (1995)
+   *   <a href="https://doi.org/10.1007/BF02198293"> Computation of real or
+   *   complex elliptic integrals</a>, Numerical Algorithms 10, 13--26 (1995);
+   *   <a href="https://arxiv.org/abs/math/9409227">preprint</a>.
    * .
-   * with the additional optimizations given in http://dlmf.nist.gov/19.36.i.
+   * with the additional optimizations given in https://dlmf.nist.gov/19.36.i.
    * The computation of the Jacobi elliptic functions uses the algorithm given
    * in
    * - R. Bulirsch,
-   *   <a href="http://dx.doi.org/10.1007/BF01397975"> Numerical Calculation of
+   *   <a href="https://doi.org/10.1007/BF01397975"> Numerical Calculation of
    *   Elliptic Integrals and Elliptic Functions</a>, Numericshe Mathematik 7,
    *   78--90 (1965).
    * .
-   * The notation follows http://dlmf.nist.gov/19 and http://dlmf.nist.gov/22
+   * The notation follows https://dlmf.nist.gov/19 and https://dlmf.nist.gov/22
    *
    * Example of use:
    * \include example-EllipticFunction.cpp
@@ -62,16 +63,10 @@ namespace GeographicLib {
   class GEOGRAPHICLIB_EXPORT EllipticFunction {
   private:
     typedef Math::real real;
-    static const real tol_;
-    static const real tolRF_;
-    static const real tolRD_;
-    static const real tolRG0_;
-    static const real tolJAC_;
-    enum { num_ = 13 }; // Max depth required for sncndn.  Probably 5 is enough.
+
+    enum { num_ = 13 }; // Max depth required for sncndn; probably 5 is enough.
     real _k2, _kp2, _alpha2, _alphap2, _eps;
-    mutable bool _init;
-    mutable real _Kc, _Ec, _Dc, _Pic, _Gc, _Hc;
-    bool Init() const;
+    real _kKc, _eEc, _dDc, _pPic, _gGc, _hHc;
   public:
     /** \name Constructor
      **********************************************************************/
@@ -80,10 +75,11 @@ namespace GeographicLib {
      * Constructor specifying the modulus and parameter.
      *
      * @param[in] k2 the square of the modulus <i>k</i><sup>2</sup>.
-     *   <i>k</i><sup>2</sup> must lie in (-&infin;, 1).  (No checking is
-     *   done.)
+     *   <i>k</i><sup>2</sup> must lie in (&minus;&infin;, 1].
      * @param[in] alpha2 the parameter &alpha;<sup>2</sup>.
-     *   &alpha;<sup>2</sup> must lie in (-&infin;, 1).  (No checking is done.)
+     *   &alpha;<sup>2</sup> must lie in (&minus;&infin;, 1].
+     * @exception GeographicErr if \e k2 or \e alpha2 is out of its legal
+     *   range.
      *
      * If only elliptic integrals of the first and second kinds are needed,
      * then set &alpha;<sup>2</sup> = 0 (the default value); in this case, we
@@ -91,36 +87,41 @@ namespace GeographicLib {
      * E(&phi;, \e k), and \e H(&phi;, 0, \e k) = \e F(&phi;, \e k) - \e
      * D(&phi;, \e k).
      **********************************************************************/
-    EllipticFunction(real k2 = 0, real alpha2 = 0);
+    EllipticFunction(real k2 = 0, real alpha2 = 0)
+      { Reset(k2, alpha2); }
 
     /**
      * Constructor specifying the modulus and parameter and their complements.
      *
      * @param[in] k2 the square of the modulus <i>k</i><sup>2</sup>.
-     *   <i>k</i><sup>2</sup> must lie in (-&infin;, 1).  (No checking is
-     *   done.)
+     *   <i>k</i><sup>2</sup> must lie in (&minus;&infin;, 1].
      * @param[in] alpha2 the parameter &alpha;<sup>2</sup>.
-     *   &alpha;<sup>2</sup> must lie in (-&infin;, 1).  (No checking is done.)
+     *   &alpha;<sup>2</sup> must lie in (&minus;&infin;, 1].
      * @param[in] kp2 the complementary modulus squared <i>k'</i><sup>2</sup> =
-     *   1 &minus; <i>k</i><sup>2</sup>.
+     *   1 &minus; <i>k</i><sup>2</sup>.  This must lie in [0, &infin;).
      * @param[in] alphap2 the complementary parameter &alpha;'<sup>2</sup> = 1
-     *   &minus; &alpha;<sup>2</sup>.
+     *   &minus; &alpha;<sup>2</sup>.  This must lie in [0, &infin;).
+     * @exception GeographicErr if \e k2, \e alpha2, \e kp2, or \e alphap2 is
+     *   out of its legal range.
      *
      * The arguments must satisfy \e k2 + \e kp2 = 1 and \e alpha2 + \e alphap2
      * = 1.  (No checking is done that these conditions are met.)  This
      * constructor is provided to enable accuracy to be maintained, e.g., when
      * \e k is very close to unity.
      **********************************************************************/
-    EllipticFunction(real k2, real alpha2, real kp2, real alphap2);
+    EllipticFunction(real k2, real alpha2, real kp2, real alphap2)
+      { Reset(k2, alpha2, kp2, alphap2); }
 
     /**
      * Reset the modulus and parameter.
      *
      * @param[in] k2 the new value of square of the modulus
-     *   <i>k</i><sup>2</sup> which must lie in (-&infin;, 1).  (No checking is
+     *   <i>k</i><sup>2</sup> which must lie in (&minus;&infin;, ].
      *   done.)
      * @param[in] alpha2 the new value of parameter &alpha;<sup>2</sup>.
-     *   &alpha;<sup>2</sup> must lie in (-&infin;, 1).  (No checking is done.)
+     *   &alpha;<sup>2</sup> must lie in (&minus;&infin;, 1].
+     * @exception GeographicErr if \e k2 or \e alpha2 is out of its legal
+     *   range.
      **********************************************************************/
     void Reset(real k2 = 0, real alpha2 = 0)
     { Reset(k2, alpha2, 1 - k2, 1 - alpha2); }
@@ -129,14 +130,15 @@ namespace GeographicLib {
      * Reset the modulus and parameter supplying also their complements.
      *
      * @param[in] k2 the square of the modulus <i>k</i><sup>2</sup>.
-     *   <i>k</i><sup>2</sup> must lie in (-&infin;, 1).  (No checking is
-     *   done.)
+     *   <i>k</i><sup>2</sup> must lie in (&minus;&infin;, 1].
      * @param[in] alpha2 the parameter &alpha;<sup>2</sup>.
-     *   &alpha;<sup>2</sup> must lie in (-&infin;, 1).  (No checking is done.)
+     *   &alpha;<sup>2</sup> must lie in (&minus;&infin;, 1].
      * @param[in] kp2 the complementary modulus squared <i>k'</i><sup>2</sup> =
-     *   1 &minus; <i>k</i><sup>2</sup>.
+     *   1 &minus; <i>k</i><sup>2</sup>.  This must lie in [0, &infin;).
      * @param[in] alphap2 the complementary parameter &alpha;'<sup>2</sup> = 1
-     *   &minus; &alpha;<sup>2</sup>.
+     *   &minus; &alpha;<sup>2</sup>.  This must lie in [0, &infin;).
+     * @exception GeographicErr if \e k2, \e alpha2, \e kp2, or \e alphap2 is
+     *   out of its legal range.
      *
      * The arguments must satisfy \e k2 + \e kp2 = 1 and \e alpha2 + \e alphap2
      * = 1.  (No checking is done that these conditions are met.)  This
@@ -173,22 +175,6 @@ namespace GeographicLib {
     Math::real alphap2() const { return _alphap2; }
     ///@}
 
-    /// \cond SKIP
-    /**
-     * @return the square of the modulus <i>k</i><sup>2</sup>.
-     *
-     * <b>DEPRECATED</b>, use k2() instead.
-     **********************************************************************/
-    Math::real m() const { return _k2; }
-    /**
-     * @return the square of the complementary modulus <i>k'</i><sup>2</sup> =
-     *   1 &minus; <i>k</i><sup>2</sup>.
-     *
-     * <b>DEPRECATED</b>, use kp2() instead.
-     **********************************************************************/
-    Math::real m1() const { return _kp2; }
-    /// \endcond
-
     /** \name Complete elliptic integrals.
      **********************************************************************/
     ///@{
@@ -197,36 +183,37 @@ namespace GeographicLib {
      *
      * @return \e K(\e k).
      *
-     * \e K(\e k) is defined in http://dlmf.nist.gov/19.2.E4
+     * \e K(\e k) is defined in https://dlmf.nist.gov/19.2.E4
      * \f[
      *   K(k) = \int_0^{\pi/2} \frac1{\sqrt{1-k^2\sin^2\phi}}\,d\phi.
      * \f]
      **********************************************************************/
-    Math::real K() const { _init || Init(); return _Kc; }
+    Math::real K() const { return _kKc; }
 
     /**
      * The complete integral of the second kind.
      *
-     * @return \e E(\e k)
+     * @return \e E(\e k).
      *
-     * \e E(\e k) is defined in http://dlmf.nist.gov/19.2.E5
+     * \e E(\e k) is defined in https://dlmf.nist.gov/19.2.E5
      * \f[
      *   E(k) = \int_0^{\pi/2} \sqrt{1-k^2\sin^2\phi}\,d\phi.
      * \f]
      **********************************************************************/
-    Math::real E() const { _init || Init(); return _Ec; }
+    Math::real E() const { return _eEc; }
 
     /**
      * Jahnke's complete integral.
      *
      * @return \e D(\e k).
      *
-     * \e D(\e k) is defined in http://dlmf.nist.gov/19.2.E6
+     * \e D(\e k) is defined in https://dlmf.nist.gov/19.2.E6
      * \f[
-     *   D(k) = \int_0^{\pi/2} \frac{\sin^2\phi}{\sqrt{1-k^2\sin^2\phi}}\,d\phi.
+     *   D(k) =
+     *   \int_0^{\pi/2} \frac{\sin^2\phi}{\sqrt{1-k^2\sin^2\phi}}\,d\phi.
      * \f]
      **********************************************************************/
-    Math::real D() const { _init || Init(); return _Dc; }
+    Math::real D() const { return _dDc; }
 
     /**
      * The difference between the complete integrals of the first and second
@@ -234,26 +221,26 @@ namespace GeographicLib {
      *
      * @return \e K(\e k) &minus; \e E(\e k).
      **********************************************************************/
-    Math::real KE() const { _init || Init(); return _k2 * _Dc; }
+    Math::real KE() const { return _k2 * _dDc; }
 
     /**
      * The complete integral of the third kind.
      *
-     * @return &Pi;(&alpha;<sup>2</sup>, \e k)
+     * @return &Pi;(&alpha;<sup>2</sup>, \e k).
      *
      * &Pi;(&alpha;<sup>2</sup>, \e k) is defined in
-     * http://dlmf.nist.gov/19.2.E7
+     * https://dlmf.nist.gov/19.2.E7
      * \f[
      *   \Pi(\alpha^2, k) = \int_0^{\pi/2}
-     *     \frac1{\sqrt{1-k^2\sin^2\phi}(1 - \alpha^2\sin^2\phi_)}\,d\phi.
+     *     \frac1{\sqrt{1-k^2\sin^2\phi}(1 - \alpha^2\sin^2\phi)}\,d\phi.
      * \f]
      **********************************************************************/
-    Math::real Pi() const { _init || Init(); return _Pic; }
+    Math::real Pi() const { return _pPic; }
 
     /**
      * Legendre's complete geodesic longitude integral.
      *
-     * @return \e G(&alpha;<sup>2</sup>, \e k)
+     * @return \e G(&alpha;<sup>2</sup>, \e k).
      *
      * \e G(&alpha;<sup>2</sup>, \e k) is given by
      * \f[
@@ -261,12 +248,12 @@ namespace GeographicLib {
      *     \frac{\sqrt{1-k^2\sin^2\phi}}{1 - \alpha^2\sin^2\phi}\,d\phi.
      * \f]
      **********************************************************************/
-    Math::real G() const { _init || Init(); return _Gc; }
+    Math::real G() const { return _gGc; }
 
     /**
      * Cayley's complete geodesic longitude difference integral.
      *
-     * @return \e H(&alpha;<sup>2</sup>, \e k)
+     * @return \e H(&alpha;<sup>2</sup>, \e k).
      *
      * \e H(&alpha;<sup>2</sup>, \e k) is given by
      * \f[
@@ -275,7 +262,7 @@ namespace GeographicLib {
      *     \,d\phi.
      * \f]
      **********************************************************************/
-    Math::real H() const { _init || Init(); return _Hc; }
+    Math::real H() const { return _hHc; }
     ///@}
 
     /** \name Incomplete elliptic integrals.
@@ -287,7 +274,7 @@ namespace GeographicLib {
      * @param[in] phi
      * @return \e F(&phi;, \e k).
      *
-     * \e F(&phi;, \e k) is defined in http://dlmf.nist.gov/19.2.E4
+     * \e F(&phi;, \e k) is defined in https://dlmf.nist.gov/19.2.E4
      * \f[
      *   F(\phi, k) = \int_0^\phi \frac1{\sqrt{1-k^2\sin^2\theta}}\,d\theta.
      * \f]
@@ -300,7 +287,7 @@ namespace GeographicLib {
      * @param[in] phi
      * @return \e E(&phi;, \e k).
      *
-     * \e E(&phi;, \e k) is defined in http://dlmf.nist.gov/19.2.E5
+     * \e E(&phi;, \e k) is defined in https://dlmf.nist.gov/19.2.E5
      * \f[
      *   E(\phi, k) = \int_0^\phi \sqrt{1-k^2\sin^2\theta}\,d\theta.
      * \f]
@@ -332,10 +319,10 @@ namespace GeographicLib {
      * @return &Pi;(&phi;, &alpha;<sup>2</sup>, \e k).
      *
      * &Pi;(&phi;, &alpha;<sup>2</sup>, \e k) is defined in
-     * http://dlmf.nist.gov/19.2.E7
+     * https://dlmf.nist.gov/19.2.E7
      * \f[
      *   \Pi(\phi, \alpha^2, k) = \int_0^\phi
-     *     \frac1{\sqrt{1-k^2\sin^2\theta}(1 - \alpha^2\sin^2\theta_)}\,d\theta.
+     *     \frac1{\sqrt{1-k^2\sin^2\theta}(1 - \alpha^2\sin^2\theta)}\,d\theta.
      * \f]
      **********************************************************************/
     Math::real Pi(real phi) const;
@@ -346,7 +333,7 @@ namespace GeographicLib {
      * @param[in] phi
      * @return \e D(&phi;, \e k).
      *
-     * \e D(&phi;, \e k) is defined in http://dlmf.nist.gov/19.2.E4
+     * \e D(&phi;, \e k) is defined in https://dlmf.nist.gov/19.2.E4
      * \f[
      *   D(\phi, k) = \int_0^\phi
      *    \frac{\sin^2\theta}{\sqrt{1-k^2\sin^2\theta}}\,d\theta.
@@ -362,19 +349,19 @@ namespace GeographicLib {
      *
      * \e G(&phi;, &alpha;<sup>2</sup>, \e k) is defined by
      * \f[
-     *   \begin{aligned}
+     *   \begin{align}
      *   G(\phi, \alpha^2, k) &=
      *   \frac{k^2}{\alpha^2} F(\phi, k) +
      *      \biggl(1 - \frac{k^2}{\alpha^2}\biggr) \Pi(\phi, \alpha^2, k) \\
      *    &= \int_0^\phi
      *     \frac{\sqrt{1-k^2\sin^2\theta}}{1 - \alpha^2\sin^2\theta}\,d\theta.
-     *   \end{aligned}
+     *   \end{align}
      * \f]
      *
      * Legendre expresses the longitude of a point on the geodesic in terms of
      * this combination of elliptic integrals in Exercices de Calcul
      * Int&eacute;gral, Vol. 1 (1811), p. 181,
-     * http://books.google.com/books?id=riIOAAAAQAAJ&pg=PA181.
+     * https://books.google.com/books?id=riIOAAAAQAAJ&pg=PA181.
      *
      * See \ref geodellip for the expression for the longitude in terms of this
      * function.
@@ -389,19 +376,20 @@ namespace GeographicLib {
      *
      * \e H(&phi;, &alpha;<sup>2</sup>, \e k) is defined by
      * \f[
-     *   \begin{aligned}
+     *   \begin{align}
      *   H(\phi, \alpha^2, k) &=
      *   \frac1{\alpha^2} F(\phi, k) +
      *        \biggl(1 - \frac1{\alpha^2}\biggr) \Pi(\phi, \alpha^2, k) \\
      *   &= \int_0^\phi
-     *     \frac{\cos^2\theta}{(1-\alpha^2\sin^2\theta)\sqrt{1-k^2\sin^2\theta}}
+     *     \frac{\cos^2\theta}
+     *          {(1-\alpha^2\sin^2\theta)\sqrt{1-k^2\sin^2\theta}}
      *     \,d\theta.
-     *   \end{aligned}
+     *   \end{align}
      * \f]
      *
      * Cayley expresses the longitude difference of a point on the geodesic in
      * terms of this combination of elliptic integrals in Phil. Mag. <b>40</b>
-     * (1870), p. 333, http://books.google.com/books?id=Zk0wAAAAIAAJ&pg=PA333.
+     * (1870), p. 333, https://books.google.com/books?id=Zk0wAAAAIAAJ&pg=PA333.
      *
      * See \ref geodellip for the expression for the longitude in terms of this
      * function.
@@ -411,14 +399,15 @@ namespace GeographicLib {
 
     /** \name Incomplete integrals in terms of Jacobi elliptic functions.
      **********************************************************************/
+    ///@{
     /**
      * The incomplete integral of the first kind in terms of Jacobi elliptic
      * functions.
      *
-     * @param[in] sn = sin&phi;
-     * @param[in] cn = cos&phi;
+     * @param[in] sn = sin&phi;.
+     * @param[in] cn = cos&phi;.
      * @param[in] dn = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
+     *   sin<sup>2</sup>&phi;).
      * @return \e F(&phi;, \e k) as though &phi; &isin; (&minus;&pi;, &pi;].
      **********************************************************************/
     Math::real F(real sn, real cn, real dn) const;
@@ -427,10 +416,10 @@ namespace GeographicLib {
      * The incomplete integral of the second kind in terms of Jacobi elliptic
      * functions.
      *
-     * @param[in] sn = sin&phi;
-     * @param[in] cn = cos&phi;
+     * @param[in] sn = sin&phi;.
+     * @param[in] cn = cos&phi;.
      * @param[in] dn = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
+     *   sin<sup>2</sup>&phi;).
      * @return \e E(&phi;, \e k) as though &phi; &isin; (&minus;&pi;, &pi;].
      **********************************************************************/
     Math::real E(real sn, real cn, real dn) const;
@@ -439,10 +428,10 @@ namespace GeographicLib {
      * The incomplete integral of the third kind in terms of Jacobi elliptic
      * functions.
      *
-     * @param[in] sn = sin&phi;
-     * @param[in] cn = cos&phi;
+     * @param[in] sn = sin&phi;.
+     * @param[in] cn = cos&phi;.
      * @param[in] dn = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
+     *   sin<sup>2</sup>&phi;).
      * @return &Pi;(&phi;, &alpha;<sup>2</sup>, \e k) as though &phi; &isin;
      *   (&minus;&pi;, &pi;].
      **********************************************************************/
@@ -452,10 +441,10 @@ namespace GeographicLib {
      * Jahnke's incomplete elliptic integral in terms of Jacobi elliptic
      * functions.
      *
-     * @param[in] sn = sin&phi;
-     * @param[in] cn = cos&phi;
+     * @param[in] sn = sin&phi;.
+     * @param[in] cn = cos&phi;.
      * @param[in] dn = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
+     *   sin<sup>2</sup>&phi;).
      * @return \e D(&phi;, \e k) as though &phi; &isin; (&minus;&pi;, &pi;].
      **********************************************************************/
     Math::real D(real sn, real cn, real dn) const;
@@ -464,10 +453,10 @@ namespace GeographicLib {
      * Legendre's geodesic longitude integral in terms of Jacobi elliptic
      * functions.
      *
-     * @param[in] sn = sin&phi;
-     * @param[in] cn = cos&phi;
+     * @param[in] sn = sin&phi;.
+     * @param[in] cn = cos&phi;.
      * @param[in] dn = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
+     *   sin<sup>2</sup>&phi;).
      * @return \e G(&phi;, &alpha;<sup>2</sup>, \e k) as though &phi; &isin;
      *   (&minus;&pi;, &pi;].
      **********************************************************************/
@@ -477,10 +466,10 @@ namespace GeographicLib {
      * Cayley's geodesic longitude difference integral in terms of Jacobi
      * elliptic functions.
      *
-     * @param[in] sn = sin&phi;
-     * @param[in] cn = cos&phi;
+     * @param[in] sn = sin&phi;.
+     * @param[in] cn = cos&phi;.
      * @param[in] dn = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
+     *   sin<sup>2</sup>&phi;).
      * @return \e H(&phi;, &alpha;<sup>2</sup>, \e k) as though &phi; &isin;
      *   (&minus;&pi;, &pi;].
      **********************************************************************/
@@ -493,82 +482,82 @@ namespace GeographicLib {
     /**
      * The periodic incomplete integral of the first kind.
      *
-     * @param[in] sn = sin&phi;
-     * @param[in] cn = cos&phi;
+     * @param[in] sn = sin&phi;.
+     * @param[in] cn = cos&phi;.
      * @param[in] dn = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
+     *   sin<sup>2</sup>&phi;).
      * @return the periodic function &pi; \e F(&phi;, \e k) / (2 \e K(\e k)) -
-     *   &phi;
+     *   &phi;.
      **********************************************************************/
     Math::real deltaF(real sn, real cn, real dn) const;
 
     /**
      * The periodic incomplete integral of the second kind.
      *
-     * @param[in] sn = sin&phi;
-     * @param[in] cn = cos&phi;
+     * @param[in] sn = sin&phi;.
+     * @param[in] cn = cos&phi;.
      * @param[in] dn = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
+     *   sin<sup>2</sup>&phi;).
      * @return the periodic function &pi; \e E(&phi;, \e k) / (2 \e E(\e k)) -
-     *   &phi;
+     *   &phi;.
      **********************************************************************/
     Math::real deltaE(real sn, real cn, real dn) const;
 
     /**
      * The periodic inverse of the incomplete integral of the second kind.
      *
-     * @param[in] stau = sin&tau;
-     * @param[in] ctau = sin&tau;
+     * @param[in] stau = sin&tau;.
+     * @param[in] ctau = sin&tau;.
      * @return the periodic function <i>E</i><sup>&minus;1</sup>(&tau; (2 \e
-     *   E(\e k)/&pi;), \e k) - &tau;
+     *   E(\e k)/&pi;), \e k) - &tau;.
      **********************************************************************/
     Math::real deltaEinv(real stau, real ctau) const;
 
     /**
      * The periodic incomplete integral of the third kind.
      *
-     * @param[in] sn = sin&phi;
-     * @param[in] cn = cos&phi;
+     * @param[in] sn = sin&phi;.
+     * @param[in] cn = cos&phi;.
      * @param[in] dn = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
-     * @return the periodic function &pi; &Pi;(&phi;, \e k) / (2 &Pi;(\e k)) -
-     *   &phi;
+     *   sin<sup>2</sup>&phi;).
+     * @return the periodic function &pi; &Pi;(&phi;, &alpha;<sup>2</sup>,
+     *   \e k) / (2 &Pi;(&alpha;<sup>2</sup>, \e k)) - &phi;.
      **********************************************************************/
     Math::real deltaPi(real sn, real cn, real dn) const;
 
     /**
      * The periodic Jahnke's incomplete elliptic integral.
      *
-     * @param[in] sn = sin&phi;
-     * @param[in] cn = cos&phi;
+     * @param[in] sn = sin&phi;.
+     * @param[in] cn = cos&phi;.
      * @param[in] dn = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
+     *   sin<sup>2</sup>&phi;).
      * @return the periodic function &pi; \e D(&phi;, \e k) / (2 \e D(\e k)) -
-     *   &phi;
+     *   &phi;.
      **********************************************************************/
     Math::real deltaD(real sn, real cn, real dn) const;
 
     /**
      * Legendre's periodic geodesic longitude integral.
      *
-     * @param[in] sn = sin&phi;
-     * @param[in] cn = cos&phi;
+     * @param[in] sn = sin&phi;.
+     * @param[in] cn = cos&phi;.
      * @param[in] dn = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
+     *   sin<sup>2</sup>&phi;).
      * @return the periodic function &pi; \e G(&phi;, \e k) / (2 \e G(\e k)) -
-     *   &phi;
+     *   &phi;.
      **********************************************************************/
     Math::real deltaG(real sn, real cn, real dn) const;
 
     /**
      * Cayley's periodic geodesic longitude difference integral.
      *
-     * @param[in] sn = sin&phi;
-     * @param[in] cn = cos&phi;
+     * @param[in] sn = sin&phi;.
+     * @param[in] cn = cos&phi;.
      * @param[in] dn = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
+     *   sin<sup>2</sup>&phi;).
      * @return the periodic function &pi; \e H(&phi;, \e k) / (2 \e H(\e k)) -
-     *   &phi;
+     *   &phi;.
      **********************************************************************/
     Math::real deltaH(real sn, real cn, real dn) const;
     ///@}
@@ -589,115 +578,133 @@ namespace GeographicLib {
     /**
      * The &Delta; amplitude function.
      *
-     * @param[in] sn sin&phi;
-     * @param[in] cn cos&phi;
+     * @param[in] sn sin&phi;.
+     * @param[in] cn cos&phi;.
      * @return &Delta; = sqrt(1 &minus; <i>k</i><sup>2</sup>
-     *   sin<sup>2</sup>&phi;)
+     *   sin<sup>2</sup>&phi;).
      **********************************************************************/
-    Math::real Delta(real sn, real cn) const
-    { return std::sqrt(_k2 < 0 ? 1 - _k2 * sn*sn : _kp2 + _k2 * cn*cn); }
+    Math::real Delta(real sn, real cn) const {
+      using std::sqrt;
+      return sqrt(_k2 < 0 ? 1 - _k2 * sn*sn : _kp2 + _k2 * cn*cn);
+    }
     ///@}
 
     /** \name Symmetric elliptic integrals.
      **********************************************************************/
     ///@{
     /**
-     * Symmetric integral of the first kind <i>R<sub>F</sub></i>.
+     * Symmetric integral of the first kind <i>R</i><sub><i>F</i></sub>.
      *
      * @param[in] x
      * @param[in] y
      * @param[in] z
-     * @return <i>R<sub>F</sub></i>(\e x, \e y, \e z)
+     * @return <i>R</i><sub><i>F</i></sub>(\e x, \e y, \e z).
      *
-     * <i>R<sub>F</sub></i> is defined in http://dlmf.nist.gov/19.16.E1
+     * <i>R</i><sub><i>F</i></sub> is defined in https://dlmf.nist.gov/19.16.E1
      * \f[ R_F(x, y, z) = \frac12
-     *       \int_0^\infty\frac1{\sqrt{(t + x) (t + y) (t + z)}}\, dt \f]
-     * If one of the arguments is zero, it is more efficient to call the
-     * two-argument version of this function with the non-zero arguments.
+     *       \int_0^\infty\frac1{\sqrt{(t + x) (t + y) (t + z)}}\, dt, \f]
+     * where at most one of arguments, \e x, \e y, \e z, can be zero and those
+     * arguments that are nonzero must be positive.  If one of the arguments is
+     * zero, it is more efficient to call the two-argument version of this
+     * function with the non-zero arguments.
      **********************************************************************/
     static real RF(real x, real y, real z);
 
     /**
-     * Complete symmetric integral of the first kind, <i>R<sub>F</sub></i> with
-     * one argument zero.
+     * Complete symmetric integral of the first kind,
+     * <i>R</i><sub><i>F</i></sub> with one argument zero.
      *
      * @param[in] x
      * @param[in] y
-     * @return <i>R<sub>F</sub></i>(\e x, \e y, 0)
+     * @return <i>R</i><sub><i>F</i></sub>(\e x, \e y, 0).
+     *
+     * The arguments \e x and \e y must be positive.
      **********************************************************************/
     static real RF(real x, real y);
 
     /**
-     * Degenerate symmetric integral of the first kind <i>R<sub>C</sub></i>.
+     * Degenerate symmetric integral of the first kind
+     * <i>R</i><sub><i>C</i></sub>.
      *
      * @param[in] x
      * @param[in] y
-     * @return <i>R<sub>C</sub></i>(\e x, \e y) = <i>R<sub>F</sub></i>(\e x, \e
-     *   y, \e y)
+     * @return <i>R</i><sub><i>C</i></sub>(\e x, \e y) =
+     *   <i>R</i><sub><i>F</i></sub>(\e x, \e y, \e y).
      *
-     * <i>R<sub>C</sub></i> is defined in http://dlmf.nist.gov/19.2.E17
+     * <i>R</i><sub><i>C</i></sub> is defined in https://dlmf.nist.gov/19.2.E17
      * \f[ R_C(x, y) = \frac12
-     *       \int_0^\infty\frac1{\sqrt{t + x}(t + y)}\,dt \f]
+     *       \int_0^\infty\frac1{\sqrt{t + x}(t + y)}\,dt, \f]
+     * where \e x &ge; 0 and \e y > 0.
      **********************************************************************/
     static real RC(real x, real y);
 
     /**
-     * Symmetric integral of the second kind <i>R<sub>G</sub></i>.
+     * Symmetric integral of the second kind <i>R</i><sub><i>G</i></sub>.
      *
      * @param[in] x
      * @param[in] y
      * @param[in] z
-     * @return <i>R<sub>G</sub></i>(\e x, \e y, \e z)
+     * @return <i>R</i><sub><i>G</i></sub>(\e x, \e y, \e z).
      *
-     * <i>R<sub>G</sub></i> is defined in Carlson, eq 1.5
+     * <i>R</i><sub><i>G</i></sub> is defined in Carlson, eq 1.5
      * \f[ R_G(x, y, z) = \frac14
      *       \int_0^\infty[(t + x) (t + y) (t + z)]^{-1/2}
      *        \biggl(
      *             \frac x{t + x} + \frac y{t + y} + \frac z{t + z}
-     *        \biggr)t\,dt \f]
-     * See also http://dlmf.nist.gov/19.16.E3.
-     * If one of the arguments is zero, it is more efficient to call the
-     * two-argument version of this function with the non-zero arguments.
+     *        \biggr)t\,dt, \f]
+     * where at most one of arguments, \e x, \e y, \e z, can be zero and those
+     * arguments that are nonzero must be positive.  See also
+     * https://dlmf.nist.gov/19.23.E6_5.  If one of the arguments is zero, it
+     * is more efficient to call the two-argument version of this function with
+     * the non-zero arguments.
      **********************************************************************/
     static real RG(real x, real y, real z);
 
     /**
-     * Complete symmetric integral of the second kind, <i>R<sub>G</sub></i>
-     * with one argument zero.
+     * Complete symmetric integral of the second kind,
+     * <i>R</i><sub><i>G</i></sub> with one argument zero.
      *
      * @param[in] x
      * @param[in] y
-     * @return <i>R<sub>G</sub></i>(\e x, \e y, 0)
+     * @return <i>R</i><sub><i>G</i></sub>(\e x, \e y, 0).
+     *
+     * The arguments \e x and \e y must be positive.
      **********************************************************************/
     static real RG(real x, real y);
 
     /**
-     * Symmetric integral of the third kind <i>R<sub>J</sub></i>.
+     * Symmetric integral of the third kind <i>R</i><sub><i>J</i></sub>.
      *
      * @param[in] x
      * @param[in] y
      * @param[in] z
      * @param[in] p
-     * @return <i>R<sub>J</sub></i>(\e x, \e y, \e z, \e p)
+     * @return <i>R</i><sub><i>J</i></sub>(\e x, \e y, \e z, \e p).
      *
-     * <i>R<sub>J</sub></i> is defined in http://dlmf.nist.gov/19.16.E2
+     * <i>R</i><sub><i>J</i></sub> is defined in https://dlmf.nist.gov/19.16.E2
      * \f[ R_J(x, y, z, p) = \frac32
-     *       \int_0^\infty[(t + x) (t + y) (t + z)]^{-1/2} (t + p)^{-1}\, dt \f]
+     *       \int_0^\infty
+     *       [(t + x) (t + y) (t + z)]^{-1/2} (t + p)^{-1}\, dt, \f]
+     * where \e p > 0, and \e x, \e y, \e z are nonnegative with at most one of
+     * them being 0.
      **********************************************************************/
     static real RJ(real x, real y, real z, real p);
 
     /**
-     * Degenerate symmetric integral of the third kind <i>R<sub>D</sub></i>.
+     * Degenerate symmetric integral of the third kind
+     * <i>R</i><sub><i>D</i></sub>.
      *
      * @param[in] x
      * @param[in] y
      * @param[in] z
-     * @return <i>R<sub>D</sub></i>(\e x, \e y, \e z) = <i>R<sub>J</sub></i>(\e
-     *   x, \e y, \e z, \e z)
+     * @return <i>R</i><sub><i>D</i></sub>(\e x, \e y, \e z) =
+     *   <i>R</i><sub><i>J</i></sub>(\e x, \e y, \e z, \e z).
      *
-     * <i>R<sub>D</sub></i> is defined in http://dlmf.nist.gov/19.16.E5
+     * <i>R</i><sub><i>D</i></sub> is defined in https://dlmf.nist.gov/19.16.E5
      * \f[ R_D(x, y, z) = \frac32
-     *       \int_0^\infty[(t + x) (t + y)]^{-1/2} (t + z)^{-3/2}\, dt \f]
+     *       \int_0^\infty[(t + x) (t + y)]^{-1/2} (t + z)^{-3/2}\, dt, \f]
+     * where \e x, \e y, \e z are positive except that at most one of \e x and
+     * \e y can be 0.
      **********************************************************************/
     static real RD(real x, real y, real z);
     ///@}
